#include "libc.h"
#include "minimp3.h"
#include "mp3wave.h"

#define MP3_BUFFER_COUNT    8

mp3_decoder_t mp3;
mp3_info_t info;
unsigned char *stream_pos;
int bytes_left;
int byte_count;
int bytes_written;
signed short sample_buffer[MP3_MAX_SAMPLES_PER_FRAME * MP3_BUFFER_COUNT];

int tmp_frames_cnt = 0;
unsigned char* tmp_buffer = NULL;

void mp3_init_vals()
{
    mp3 = NULL;
    memset( &info, 0 , sizeof( mp3_info_t ) );
    stream_pos = NULL;
    bytes_left = 0;
    byte_count = 0;
    bytes_written = 0;
    memset( &sample_buffer[0], 0, MP3_MAX_SAMPLES_PER_FRAME * MP3_BUFFER_COUNT );

    if ( tmp_buffer != NULL )
    {
        free( tmp_buffer );
        tmp_buffer = NULL;
    }
}

int convert_mp3_to_wave( const unsigned char* src, int srcsize, unsigned char** out, mp3track_info* oinfo )
{
    int retlen = 0;
#ifdef DEBUG
    int testque = 0;
#endif

    if ( src == NULL )
        return -1;

    if ( srcsize <= 0 )
        return -2;

    mp3_init_vals();

    mp3 = mp3_create();

    bytes_left = srcsize - 128;

    stream_pos = (unsigned char*)src;

    byte_count = mp3_decode( mp3, stream_pos, bytes_left, (signed short*)sample_buffer, &info );
    if ( byte_count == 0 )
    {
        mp3_done( mp3 );
        mp3_init_vals();
        return -3;
    }

    if ( oinfo != NULL )
    {
        memcpy( oinfo, &info, sizeof(mp3track_info) );
    }

    tmp_frames_cnt = ( srcsize / byte_count ) + 1;
    if ( tmp_frames_cnt <= 0 )
    {
        mp3_done( mp3 );
        mp3_init_vals();
        return -4;
    }

    int allocsz = info.audio_bytes * tmp_frames_cnt;

#ifdef DEBUG
    printf("first frame size = %d\n", byte_count );
    printf("temp frame count = %d\n", tmp_frames_cnt );
    printf("temp audio byte  = %d\n", info.audio_bytes );
    printf("allocating .. %d bytes .. ", allocsz );
#endif
    tmp_buffer = (unsigned char*)malloc( allocsz );
#ifdef DEBUG
    if( tmp_buffer != NULL )
    {
        printf("Ok.\n");
    }
    else
    {
        printf("Fail !\n");

        mp3_done( mp3 );
        mp3_init_vals();
        return -5;
    }

    testque = 1;
#else
    if ( tmp_buffer == NULL )
    {
        mp3_done( mp3 );
        mp3_init_vals();
        return -5;
    }
#endif

#ifdef DEBUG
    printf("decoding mp3 ...\n");
#endif

    memcpy( &tmp_buffer[ bytes_written ], sample_buffer, info.audio_bytes );
    bytes_written += info.audio_bytes;

#ifdef DEBUG
    printf("\rbytes written = %d bytes, mp3 left = %d ", bytes_written, bytes_left);
#endif

    while( byte_count > 0 )
    {
        stream_pos += byte_count;
        bytes_left -= byte_count;

        byte_count = mp3_decode( mp3, stream_pos, bytes_left, (signed short*)sample_buffer, &info);

        if( allocsz < bytes_written + info.audio_bytes )
        {
            allocsz *= 2;
#ifdef DEBUG
            printf("\nre-allocating .. %d bytes .. \n", allocsz );
#endif
            tmp_buffer = libc_realloc( tmp_buffer, allocsz );
        }

        memcpy( &tmp_buffer[ bytes_written ], sample_buffer, info.audio_bytes );
        bytes_written += info.audio_bytes;

#ifdef DEBUG
        testque++;
        printf("\r(%03d) bytes written = %d bytes, mp3 left = %d ", testque, bytes_written, bytes_left);
#endif

    }

#ifdef DEBUG
    printf("\nWriting to out buffer ...");
#endif
    *out = (unsigned char*)malloc( bytes_written + 1 );
    if ( *out != NULL )
    {
        memcpy( *out, tmp_buffer, bytes_written );
    }
#ifdef DEBUG
    printf("done\n");
#endif

    retlen = bytes_written;

    mp3_done( mp3 );
    mp3_init_vals();

    return retlen;
}
